// AcquireFormat.h
//
// Copyright (c) 1999 Bruxton Corporation
//
// Author: Joseph Grote

// Bruxton Corporation Acquire data format.

// The file is written with Intel byte ordering.

// This file format is designed so that all values are 8 byte aligned.

// Data is always written to the file in multiples of ACQUIRE_BLOCK_SIZE.

// The data file has a hierarchical structure, organized as series, sweeps
// and segments. Within segments there are channels and data blocks.

// The file format is as follows:

//*****************************************************************
// header
// file prefix block
// sweep prefix block
// data block
// data block
//		.
//		.
// sweep block
// sweep prefix block
// data block
//		.
//		.
// series block
// sweep block
//		.
//		.
//		.
// series block
// control block
// header
// AcquirePrefix
//*****************************************************************

// the header contains 
// AcquirePrefix
// AcquireHeader

// AcquirePrefix is duplicated at the end of the file in an ACQUIRE_BLOCK_SIZE size block
// in order to be consistent with previous versions of the file format.
// DataAccess first checks the the last block to determine the file type and version.

// The file prefix block contains:

// array of channel records, grouped in the following order:

// AcquireAnalogInChannelRecords
// AcquireAperiodicChannelRecords

// A series block contains:

// AcquireSeriesDescriptor
// directory of sweep prefix block offsets within the control block.

// a sweep prefix block contains:

// array of analog_in_channel_count AcquireSweepPrefixAnalogChannel
// array of aperiodic_channel_count AcquireSweepPrefixAperiodicChannel

// a data block contains a block of data.

// a sweep block contains:

// AcquireSweepDescriptor
// AcquireChannelDescriptor	for the control channels
// AcquireBlockDescriptors for the control channels
// AcquireChannelDescriptor	for the control channel extended data
// AcquireBlockDescriptors for the control channel extended data
// Segment directory listing AcquireSegmentDescriptor offsets within the control block
// AcquireSegmentDescriptor
// Channel directory listing AcquireChannelDescriptor offsets within the control block
//    for the analog input channels
// AcquireChannelDescriptor
// AcquireBlockDescriptors
//		.
//		.
//		.
// AcquireSegmentDescriptor
// Channel directory
// AcquireChannelDescriptor
// AcquireBlockDescriptors
//		.
//		.
//		.

// the control block contains:

// array of channel records, grouped in the following order:

// AcquireAnalogInChannelRecords
// AcquireAperiodicChannelRecords

// repeat of series, sweep prefix and sweep blocks (without individual block tags
// and block boundary padding).
// a directory of series block offsets within the control block

// V6.1 Set data_size in tags for all block types except the header blocks.

#ifndef AcquireFormat_H
#define AcquireFormat_H

#ifdef _WINDOWS
#ifndef int8_t
#define int8_t __int8
#endif
#ifndef int16_t
#define int16_t __int16
#endif
#ifndef int32_t
#define int32_t __int32
#endif
#ifndef int64_t
#define int64_t __int64
#endif
#else
/* Added 11-2009 Chris Nicolai: type defs that work in linux */
#ifndef int8_t
#define int8_t char
#endif
#ifndef int16_t
#define int16_t short
#endif
#ifndef int32_t
#define int32_t int
#endif
#ifndef int64_t
#define int64_t long long
#endif

#endif

#define ACQUIRE_FILE_FORMAT_VERSION 6
#define	ACQUIRE_FILE_MINOR_VERSION 1

static const int8_t VENDOR_BRUXTON[] = {'I','n','s','t','r','u','T','E','C','H'};

// Bruxton file formats.

enum
{
	ACQUIRE_FORMAT = 1
};

#define ACQUIRE_BLOCK_SIZE 1024
#define ACQUIRE_FILE_COMMENT_SIZE 256
#define ACQUIRE_COMMENT_SIZE 80
#define ACQUIRE_FILE_TECHNIQUE_SIZE 256
#define ACQUIRE_TECHNIQUE_SIZE 80
#define ACQUIRE_SCREEN_SIZE 80
#define ACQUIRE_LABEL_SIZE 16
#define ACQUIRE_UNITS_SIZE 16
#define ACQUIRE_SIGNAL_SIZE 32
#define ACQUIRE_CONTROL_NAME_SIZE 80
#define ACQUIRE_USER_NAME_SIZE 64
#define ACQUIRE_COMPUTER_NAME_SIZE 64

// AcquirePrefix contains information to go at the head of the file.
// It contains the same fields for all file format versions.

// Note that Acquire file format version 1 uses the reserved space
// and is in Macintosh byte ordering.
// See SKALARFormatV1.h.

struct AcquirePrefix
{
	int8_t vendor[8];		// company name
	int32_t format;			// application
	int32_t version;		// version number
	int64_t minor_version;
	int64_t reserved[3];
};

#ifndef __cplusplus
typedef struct AcquirePrefix AcquirePrefix;
#endif

// AcquireHeader

struct AcquireHeader
{

	int64_t date_time;			// date/time created in 100 ns units from Jan 1 1601
	int64_t file_tag;			// file tag used for reconstruction of damaged data
									// use the same value in mBlockTag				
	int16_t comment[ACQUIRE_FILE_COMMENT_SIZE];
	int16_t technique[ACQUIRE_FILE_TECHNIQUE_SIZE];
	double clock_rate;				// base clock rate in Hz
	int64_t frame_interval;		// frame interval in clock ticks
	int64_t analog_in_channel_count;
	int64_t analog_in_maximum_block_size;	// maximum possible size of a data block in bytes
	int64_t analog_in_maximum_data_count; 	// maximum data values per data block
	int64_t aperiodic_channel_count;
	int64_t aperiodic_maximum_block_size;	// maximum possible size of a data block in bytes
	int64_t series_count;
	int64_t file_size;			// file size in bytes
	int64_t control_size;		// size of the control block in bytes
	int64_t control_position;		// position of control block in the file
	int64_t directory_position;		// series directory offset in control block
	int16_t user_name[ACQUIRE_USER_NAME_SIZE];	// Current logged in user.
	int16_t computer_name[ACQUIRE_USER_NAME_SIZE];	// Current logged in user.
};

#ifndef __cplusplus
typedef struct AcquireHeader AcquireHeader;
#endif

// AcquireSeriesDescriptor

struct AcquireSeriesDescriptor
{
	int16_t comment[ACQUIRE_COMMENT_SIZE];
	int16_t technique[ACQUIRE_TECHNIQUE_SIZE];
	int64_t start_time;		// time of start of series 100 ns units from Jan 1 1601 
	int64_t sweep_count;
};

#ifndef __cplusplus
typedef struct AcquireSeriesDescriptor AcquireSeriesDescriptor;
#endif

// AcquireSweepPrefixAnalogChannel

// Channel sweep prefix for analog channels.
// Gain is the conversion from V to the SI units representing an external amplifier.
// In version 6.0 offset is a place holder for potential amplifiers that supply an offset.
// It should always be 0.

struct AcquireSweepPrefixAnalogChannel
{	
	double maximum_voltage;
	double minimum_voltage;
	int64_t maximum_value;
	int64_t minimum_value;
	double gain;
	double offset;
};

#ifndef __cplusplus
typedef struct AcquireSweepPrefixAnalogChannel AcquireSweepPrefixAnalogChannel;
#endif

// AcquireSweepPrefixAperiodicChannel

// Channel sweep prefix for control channels.
// value is the control holding value.

struct AcquireSweepPrefixAperiodicChannel
{	
	double value;
	int64_t reserved[4];
};

#ifndef __cplusplus
typedef struct AcquireSweepPrefixAperiodicChannel AcquireSweepPrefixAperiodicChannel;
#endif

// AcquireSweepDescriptor 

struct AcquireSweepDescriptor
{
	int16_t comment[ACQUIRE_COMMENT_SIZE];
	int16_t technique[ACQUIRE_TECHNIQUE_SIZE];
	int16_t screen[ACQUIRE_SCREEN_SIZE];
	int64_t start_time;		// time of start of sweep 100 ns units from Jan 1 1601 
	int64_t segment_count;
	int64_t duration;		// sweep duration in frames
	int64_t control_data_location;	// Location of the control data channel descriptor in the control block.
	int64_t segment_location;	// Location of the segment directory in the control block.
};

#ifndef __cplusplus
typedef struct AcquireSweepDescriptor AcquireSweepDescriptor;
#endif

// AcquireSegmentDescriptor

struct AcquireSegmentDescriptor
{
	int64_t start;		// start time in frames from the start of the sweep
	int64_t duration;	// segment duration in frames
};

#ifndef __cplusplus
typedef struct AcquireSegmentDescriptor AcquireSegmentDescriptor;
#endif

// AcquireChannelDescriptor

struct AcquireChannelDescriptor
{
	int64_t block_count;
};

#ifndef __cplusplus
typedef struct AcquireChannelDescriptor AcquireChannelDescriptor;
#endif

// AcquireBlockDescriptor

struct AcquireBlockDescriptor
{
	int64_t size;		// block size in bytes
	int64_t data_count;
	int64_t data_size;  // data size in bytes
	int64_t location;	// data block location within the file
};

#ifndef __cplusplus
typedef struct AcquireBlockDescriptor AcquireBlockDescriptor;
#endif

// AcquireAnalogInChannelRecord

// Channel record for analog input channels.

// natural_scale is the power of 1000 to multiply the units to get the natural units.
// E.g., if units are A and the natural units are pA, natural_scale is -4.

struct AcquireAnalogInChannelRecord
{
	int16_t label[ACQUIRE_LABEL_SIZE];	// physical channel identification in wide chars
	int16_t signal[ACQUIRE_SIGNAL_SIZE];	// channel description in wide chars
	int16_t units[ACQUIRE_UNITS_SIZE];	// SI units of the data.
	int64_t natural_scale;	// natural_scale scale conversion.
};

#ifndef __cplusplus
typedef struct AcquireAnalogInChannelRecord AcquireAnalogInChannelRecord;
#endif

// AcquireAperiodicChannelRecord

struct AcquireAperiodicChannelRecord
{
	int64_t type;
	int16_t name[ACQUIRE_CONTROL_NAME_SIZE];
	int16_t description[ACQUIRE_COMMENT_SIZE];
	int16_t signal[ACQUIRE_SIGNAL_SIZE];
	int16_t units[ACQUIRE_UNITS_SIZE];
	int64_t natural_scale;	// natural_scale scale conversion.
	int64_t reserved[8];
};

#ifndef __cplusplus
typedef struct AcquireAperiodicChannelRecord AcquireAperiodicChannelRecord;
#endif

// Aperiodic channel types.

enum
{
	ACQUIRE_APERIODIC_CHANNEL_CONTROL = 0,
	ACQUIRE_APERIODIC_CHANNEL_IMAGE
};

// AcquireAperiodicEntry

// Entry for an aperidic channel.
// The entry may contain an attached text string, data and extended data.
// The int16_t text string immediately follows the marker in memory.
// The text string is zero terminated.
// The data immediately follows the text in memory.
// The extended data is stored as a block in the extended data channel.

struct AcquireAperiodicEntry
{
	int64_t entry_size;		// size in bytes including the text string and data.
	int64_t control_channel;
	int64_t frame;
	double value;
	int64_t text_size;	// size of the text string in characters including the terminating zero.
	int64_t data_size;	// size of the data in bytes.
	int64_t extended_data_size;	// size of the extended data in bytes.
	int64_t extended_block_index; // block index of the extended data.
};

#ifndef __cplusplus
typedef struct AcquireAperiodicEntry AcquireAperiodicEntry;
#endif

// mBlockTag

// A tag to prepend all blocks to be used in reconstruction
// of damaged data files.

struct AcquireBlockTag
{
	int64_t file_tag;		// same as AcquireHeader.file_tag
	int64_t block_index;
	int64_t block_type;		// block type, enumerated below
	int64_t block_size;		// block size in bytes
	int64_t data_count;
	int64_t data_size;		// data size in bytes
	int64_t	channel;
	int64_t frame;
};

#ifndef __cplusplus
typedef struct AcquireBlockTag AcquireBlockTag;
#endif

// Block types.

enum
{
	ACQUIRE_DATA_BLOCK = 1,
	ACQUIRE_SWEEP_BLOCK,
	ACQUIRE_CONTROL_BLOCK,
	ACQUIRE_SERIES_BLOCK,
	ACQUIRE_FILE_PREFIX_BLOCK,
	ACQUIRE_SWEEP_PREFIX_BLOCK
};

#endif
